{{

______________________________________________________________________________
********************************************************* File starts here ***
Zerocat Chipflasher --- Flash free firmware, kick the Management Engine.

Copyright (C) 2020, 2021, 2022, 2024, 2025  Kai Mertens <kmx@posteo.net>

File kick2.spin --- The top object file for this firmware2 project.

This file is part of Zerocat Chipflasher.

See end of file for terms of use.

******************************************************************************
}}



{{


IMPORTANT NOTICE
================


* On PCBv1 aka board-v1, the SPI CE# line is pulled up via trimmer.

    Adjust a reasonable value, not too small.

* On PCBv2.0 aka board-v2.0, the SPI CE# line is pulled up via R23 and R10.

    These resistors are populated with 91 Ohm each, forming a pull-up
    value of 45.5 Ohm on the first series of devices, shipped in 2024 and
    earlier.

    That is too small!

    (On the chip flasher development board, this pull-up is set to
    47.62 Ohm, which works in test setups with hand-made wiring.)

    As a workaround, cut one end of R10 with a pince and twist the
    resistor aside, carefully. The remaining pull-up value is 91 Ohm
    (the value of R23), which should be a reasonable value for most use
    cases.

    In 2025, the PCB will provide sockets for R23 and R10, such that
    this pull-up can be adjusted. However, the formed resistor value
    must not be smaller than 42 Ohm, otherwise the Propeller pins 0 and
    1 will be damaged.

    These equations apply:

                    1                  1         1
        R_pull  =  ---  ,      x  =  -----  +  -----
                    x                 R23       R10

* On PCBv2.1 aka board-v2.1, the SPI CE# line is pulled up via trimmer,
  which is digitally monitored.

    In 2025, the PCBv2.1 candidate for FSF Respect Your Freedom (RYF)
    certification uses R23 and R10 as an offset for the trimmer, to
    protect Propeller pins even if the trimmer would be adjusted to zero.

    These equations apply:

                               1
        R_pull  =  R_trim  +  ---    ,
                               x

        R_trim  =  0..500 (approx.)  ,

                     1         1
             x  =  -----  +  -----
                    R23       R10

    The value of R_pull is displayed in the menu of `kick2-connect`.
    Upon change, the display should be refreshed by user's key input.

    A value of 91 Ohm is still considered as a reasonable value for
    many use cases, however, some targets require an adjustment.


}}



''
''
''# Object Configuration (set via Makefile)
''
''Tell us which interface to use:
''
''* connect
''* flashrom
''* terminal
''* none
'
'
'   Maintain the configuration token:
'
'   * enable : "^# \t "
'   * disable: "^'~ #"
'
'
# 	   define _CFG_connect
'~ #   define _CFG_flashrom
'~ #   define _CFG_terminal
'~ #   define _CFG_nointerface



''
''
''# Interface Description
OBJ     'This directive triggers the compiler to insert a summary.
''
''
''# Loaded Objects
''


        'objects without VAR block
        pcb             : "PCB"                 ''Load PCB configuration object.
        asc             : "ASCII"               ''Load ASCII object.
        time            : "time"                ''Load time object.
        error           : "error"               ''Load error object.


        'objects with VAR block
#   ifdef _CFG_connect
        menu            : "menu"                ''Load menu object.
#   endif
#   ifdef _CFG_terminal
        menu            : "menu"                ''Load menu object.
#   endif
#   ifdef _CFG_flashrom
        serprog         : "serprog"             ''Load serprog object.
#   endif



''
''
''# Constants
''
CON

        ''* DIP Switches A and B: Board Version
        ''
        ''    Devkit: open = H, closed straight = L, closed angular = H
        ''
        ''    open = H, closed = L
        ''
        ''    Board Version  |  SWDIP_B  |  SWDIP_A
        ''    ---------------|-----------|-----------
        ''             V1    |        L  |        L
        ''             V2    |        L  |        H
        ''             V2.1  |        L  |        H_delayed
        ''             V3    |        H  |        L
        ''             V4    |        H  |        H
        ''
        ''    On PCBv1, SWDIP_A is connected to GND via resistor.
        ''
        ''    On PCBv1, SWDIP_B is not connected but will hopefully read low at start.
        ''
        ''    On PCBv2.1, SWDIP_A is connected to an RC network.
        ''    It can be detected by driving SWDIP_A low and then waiting for a low-to-high transition.
        ''
        SWDIP_A                 = 16
        SWDIP_B                 = 17


        ''* LED Pins
        D1      = 15
        D2      = 14
        D3      = 13


        ''* System Speed
        _XINFREQ            = 5_000_000                         '_XINFREQ  --- External Crystal Frequency of 5MHz
        _CLKMODE            = XTAL1 + PLL16X                    '_CLKMODE  --- Low Speed Crystal x 16    = 80MHz
        '_CLKFREQ is calculated automatically from above settings


        ''* Basic Timing
        CYCL_MIN            = 381
        HANDSHAKE_TIMEOUT   = 25                                '25 seconds login time, no reset possible!


        ''* Project's Memory Usage
        '
        'Reserve memory to avoid application failure during runtime.
        '_STACK and _FREE must sum up to under MAX_TOTAL longs.
        'Set _STACK and/or _FREE to maximum in order to get informative compile error.
        '
        MAX_TOTAL           = 8192                              'MAX_TOTAL
        MAX_STACK           = 4096                              'MAX_STACK
'        _STACK              = MAX_STACK                         'Test _STACK Maximum
'        _FREE               = MAX_TOTAL - MAX_STACK             'Test _FREE Maximum
        _STACK              = 760                               '_STACK
        _FREE               = 760                               '_FREE



''
''
''# Functions
'


PUB     kick2 | errcode, vboard, baudrate, cfg, RST_inhibit

        '' This is the Chipflasher's new firmware, coded in Spin/PASM.

        'get board configuration
        vboard := get_board_version
        RST_inhibit := pcb.get_RST_INHIBIT(vboard)
        cfg := pcb.get_board_config(vboard)
        baudrate := pcb.get_baudrate(cfg)

        'init
        DIRA[RST_inhibit]~~
        DIRA[D1..D3]~~

        'hello world! (duration: 0.5s .. 3.6s)
        CASE baudrate
            38400:    errcode := error#ERR_HELLO_38400
            57600:    errcode := error#ERR_HELLO_57600
            OTHER:    errcode := error#ERR_HELLO
        REPEAT vboard & $0f
            error.display(errcode)

        'additional time window for initial resets: 3 seconds
        WAITCNT(CLKFREQ * 3 + CNT)

        'approximately 3.5s .. 6.6s after start: disable RST until we are done
        OUTA[RST_inhibit]~~                             'disable pin RST
        errcode := error#ERR_RST
        error.display(errcode)                          'display status

        'start interface
#   ifdef _CFG_flashrom
        IF pcb.check_cable_SPI
            'start communication: try flashrom utility
            errcode := serprog.start(vboard, cfg, baudrate)
        ELSE
            errcode := error#ERR_CABLE_SPI
#   endif
#   ifdef _CFG_connect
        IF pcb.check_cable_SPI
            'start communication: try connect utility
            errcode := menu.handshake(baudrate, HANDSHAKE_TIMEOUT)
            IF errcode == error#ERR_SUCCESS
                OUTA[RST_inhibit]~                          'release RST_inhibit
                errcode := menu.start(vboard, cfg, baudrate, menu#MENU_CONNECT)
        ELSE
            errcode := error#ERR_CABLE_SPI
#   endif
#   ifdef _CFG_terminal
        IF pcb.check_cable_SPI
            'start communication: assume propeller terminal
            errcode := menu.start(vboard, cfg, baudrate, menu#MENU_TERMINAL)
        ELSE
            errcode := error#ERR_CABLE_SPI
#   endif
#   ifndef _CFG_flashrom
#       ifndef _CFG_connect
#           ifndef _CFG_terminal
        errcode := error#ERR_INTERFACE
#           endif
#       endif
#   endif


        'end of program:
        IF pcb.is_monitor_RST(vboard)
            'set up Counter B for Pos. Edge Detection on RS232_RST
            FRQB := 1
            CTRB := %0_01010_000 << 23 + 0 << 9 + pcb#RS232_RST
            PHSB := 0

            IF INA[pcb#RS232_RST]
                OUTA[RST_inhibit]~

            REPEAT
                IF PHSB
                    OUTA[RST_inhibit]~
                    errcode := error#ERR_HANDSHAKE
                error.display(errcode)
        ELSE
            'display status at least three times in case RESn is active
            REPEAT 3
                error.display(errcode)
            OUTA[RST_inhibit]~
            REPEAT
                error.display(errcode)


PRI     get_board_version : vboard | sw

        '' Retrieve board version from PCB, read SWDIP_A and SWDIP_B:
        ''
        ''    Devkit: open = H, closed straight = L, closed angular = H
        ''
        ''    open = H, closed = L
        ''
        ''    SWDIP_B  |  SWDIP_A     | PCB version   | Return Value
        ''    ---------|--------------|---------------|---------------
        ''          L  |  L           | board-v1      | $01
        ''          L  |  H           | board-v2.0    | $02
        ''          L  |  H_delayed   | board-v2.1    | $12
        ''          H  |  L           | board-v3      | $03
        ''          H  |  H           | board-v4      | $04
        ''
        ''    On PCBv1, SWDIP_A is connected to GND via resistor.
        ''
        ''    On PCBv1, SWDIP_B is not connected but will hopefully read low at start.
        ''
        ''    On PCBv2.1, SWDIP_A is connected to an RC network.
        ''    It can be detected by driving SWDIP_A low and then
        ''    waiting for a low-to-high transition within 10 millseconds.

        'drive SWDIP_A low
        DIRA[SWDIP_A]~~
        time.sleep_10us

        'read SWDIP_A, sample #0
        DIRA[SWDIP_A]~
        sw := INA[SWDIP_A] << 1

        'get board version
        time.sleep_msec(5)
        sw |= INA[SWDIP_A]
        vboard := INA[SWDIP_B..SWDIP_A] + 1
        IF sw == %01
            vboard |= 1 << 4       'set v2 subversion



{{


******************************************************************************
Terms of Use:

Zerocat Chipflasher is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

Zerocat Chipflasher is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with Zerocat Chipflasher.  If not, see <http://www.gnu.org/licenses/>.

************************************************************** End of File ***
}}
